#ifndef ROOT_InputParameters
#define ROOT_InputParameters

#include <map>
#include <string>

#include "TNamed.h"
#include "TString.h"

class InputParameters : public TNamed {
private:
   TString			 fString;
   Int_t                         fInt;       // An Int_t parameter
   Double_t                      fDouble;    // A Double_t parameter
   Bool_t                        fBool;      // A Bool_t parameter
   std::map<std::string,TString>       fMapString; // A map of named strings
   std::map<std::string,int>           fMapInt;    // A map of named ints
   std::map<std::string,double>        fMapDouble; // A map of named doubles
   std::map<std::string,bool>          fMapBool;   // A map of named booleans

public:
   InputParameters(const char *s = "Set Of Parameters") : TNamed(s,""), fString(""), fInt(0), fDouble(0.), fBool(kFALSE) { }
   virtual ~InputParameters() { }

   // Getters
   const char* GetString() const { return fString.Data(); }
   Int_t TheInt() const { return fInt; }
   Double_t TheDouble() const { return fDouble; }
   Bool_t TheBool() const { return fBool; }
   const char *TheNamedString(std::string n) {
      std::map<std::string,TString>::iterator iter = fMapString.find(n);
      if (iter != fMapString.end()) { return iter->second.Data(); }
      return NULL;
   }
   Int_t TheNamedInt(std::string n, Int_t &i) {
      std::map<std::string,int>::iterator iter = fMapInt.find(n);
      if (iter != fMapInt.end()) { i = iter->second; return 0; }
      return -1;
   }
   Int_t TheNamedDouble(std::string n, Double_t &d) {
      std::map<std::string,double>::iterator iter = fMapDouble.find(n);
      if (iter != fMapDouble.end()) { d = iter->second; return 0; }
      return -1;
   }
   Int_t TheNamedBool(std::string n, Bool_t &b) {
      std::map<std::string,bool>::iterator iter = fMapBool.find(n);
      if (iter != fMapBool.end()) { b = iter->second; return 0; }
      return -1;
   }

   // Setters
   void SetString(std::string s) { fString = s; };
   void SetInt(int i) { fInt = i; };
   void SetDouble(double d) { fDouble = d; };
   void SetBool(bool b) { fBool = b; };
   void SetNamedString(std::string n, std::string m) { fMapString[n] = m; }
   void SetNamedInt(std::string n, int i) { fMapInt[n] = i; }
   void SetNamedDouble(std::string n, double d) { fMapDouble[n] = d; }
   void SetNamedBool(std::string n, bool b) { fMapBool[n] = b; }

   // Dump content
   void DumpParms() {
      Printf(" +++++++++++++++ My Input Parameters ++++++++++++++++++++ ");
      Printf(" + Name: %s", GetName());
      Printf(" + String: %s", GetString());
      Printf(" + Int: %d, Double: %f, Bool: %d", TheInt(), TheDouble(), TheBool());
      Printf(" + Named String(s): %d", (int) fMapString.size());
      std::map<std::string,TString>::iterator its = fMapString.begin();
      while (its != fMapString.end()) { Printf(" +   '%s' -> %s", its->first.c_str(), its->second.Data()); its++; }
      Printf(" + Named Int(s): %d", (int) fMapInt.size());
      std::map<std::string,int>::iterator iti = fMapInt.begin();
      while (iti != fMapInt.end()) { Printf(" +   '%s' -> %d", iti->first.c_str(), iti->second); iti++; }
      Printf(" + Named Double(s): %d", (int) fMapDouble.size());
      std::map<std::string,double>::iterator itd = fMapDouble.begin();
      while (itd != fMapDouble.end()) { Printf(" +   '%s' -> %f", itd->first.c_str(), itd->second); itd++; }
      Printf(" + Named Bool(s): %d", (int) fMapBool.size());
      std::map<std::string,bool>::iterator itb = fMapBool.begin();
      while (itb != fMapBool.end()) { Printf(" +   '%s' -> %d", itb->first.c_str(), itb->second); itb++; }
      Printf(" +++++++++++++++++++++++++++++++++++++++++++++++++ ");
   }

   // We need to stream this class
   ClassDef(InputParameters, 1)  // Structure to define many parameters in one go
};

#endif
